@@ -1,5 +1,24 @@
 use proc_macro::*;
 
+// In order to add a method to a struct we don't own
+trait HackTrait {
+    fn next_after_visibility(&mut self) -> Option<TokenTree>;
+}
+
+impl HackTrait for proc_macro::token_stream::IntoIter {
+    fn next_after_visibility(&mut self) -> Option<TokenTree> {
+        match self.next() {
+            Some(TokenTree::Ident(ident)) if ident.to_string() == "pub" => {
+                match self.next() {
+                    Some(TokenTree::Group(group)) if group.delimiter() == Delimiter::Parenthesis => self.next(),
+                    t => t,
+                }
+            },
+            t => t,
+        }
+    }
+}
+
 #[derive(Debug)]
 pub struct FieldDesc {
     pub name: String,
@@ -15,19 +34,8 @@ pub struct StructDesc {
 pub fn read_struct(tokens: TokenStream) -> StructDesc {
     let mut tokens = tokens.into_iter();
 
-    // Visibility
-    let next_token = match tokens.next() {
-        Some(TokenTree::Ident(ident)) if ident.to_string() == "pub" => {
-            match tokens.next() {
-                Some(TokenTree::Group(group)) if group.delimiter() == Delimiter::Parenthesis => tokens.next(),
-                t => t,
-            }
-        },
-        t => t,
-    };
-
     // Struct
-    match next_token {
+    match tokens.next_after_visibility() {
         Some(TokenTree::Ident(ident)) if ident.to_string() == "struct" => (),
         _ => panic!("Expected struct"),
     };
@@ -48,7 +56,7 @@ pub fn read_struct(tokens: TokenStream) -> StructDesc {
     // Fields
     let mut fields = Vec::new();
     loop {
-        let field_name = match tokens.next() {
+        let field_name = match tokens.next_after_visibility() {
             Some(TokenTree::Ident(ident)) => ident.to_string(),
             None => break,
             _ => panic!("Expected field name"),
